'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var doubleDigits = [0, 2, 4, 6, 8, 1, 3, 5, 7, 9];
// https://tools.ietf.org/html/rfc4226

var digitsPower = [1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000];

var calcChecksum = function calcChecksum(num, digits) {
  var doubleDigit = true;
  var total = 0;

  while (0 < digits--) {
    var digit = parseInt(num % 10);
    num /= 10;
    if (doubleDigit) digit = doubleDigits[digit];
    total += digit;
    doubleDigit = !doubleDigit;
  }

  var result = total % 10;
  if (result > 0) result = 10 - result;
  return result;
};

var generate = function generate(_ref) {
  var secret = _ref.secret,
      movingFactor = _ref.movingFactor,
      codeDigits = _ref.codeDigits,
      addChecksum = _ref.addChecksum,
      truncationOffset = _ref.truncationOffset;

  if (secret) {
    if (!movingFactor) movingFactor = 0;
    if (!codeDigits) codeDigits = 6;
    if (!addChecksum) addChecksum = false;
    if (!truncationOffset) truncationOffset = -1;

    var result = '';
    var digits = addChecksum ? codeDigits + 1 : codeDigits;

    var text = Buffer.alloc(8);
    for (var i = text.length - 1; i >= 0; i--) {
      text[i] = movingFactor & 0xff;
      movingFactor >>= 8;
    }

    var hmac = _crypto2.default.createHmac('sha1', secret);
    hmac.update(text);
    var hash = Buffer.from(hmac.digest('hex'), 'hex');

    var offset = hash[hash.length - 1] & 0xf;
    if (0 <= truncationOffset && truncationOffset < hash.length - 4) offset = truncationOffset;

    var binary = (hash[offset] & 0x7f) << 24 | (hash[offset + 1] & 0xff) << 16 | (hash[offset + 2] & 0xff) << 8 | hash[offset + 3] & 0xff;

    var otp = binary % digitsPower[codeDigits];
    if (addChecksum) otp = otp * 10 + calcChecksum(otp, codeDigits);
    result = otp.toString();
    while (result.length < digits) {
      result = '0' + result;
    }return result;
  } else {
    throw Error('No secret value');
  }
};

exports.default = {
  generate: generate
};